@@ -15,14 +15,16 @@ if fc is not None:
     except FileNotFoundError:
         Conf = ConfigurationManager()
 else:
+    config_path = None
     print("Could not find configuration directory - settings will not be saved")
     Conf = ConfigurationManager()
 
+
 def save_config():
     if fc is None:
         return
     Conf.save_file(config_path)
-atexit.register(save_config)
+
 
 APP_LOCATION = str(os.path.join(os.path.dirname(os.path.realpath(__file__)), '..'))
 PLUGIN_PATH = str(os.path.join(APP_LOCATION, 'plugins'))
