@@ -1,6 +1,7 @@
 from typing import TYPE_CHECKING
 
 if TYPE_CHECKING:
+    from angrmanagement.data.library_docs import LibraryDocs
     from angrmanagement.ui.main_window import MainWindow
     from angrmanagement.plugins.plugin_manager import PluginManager
 
@@ -10,3 +11,4 @@ class GlobalInfo:
     daemon_inst = None
     daemon_conn = None
     headless_plugin_manager: 'PluginManager' = None
+    library_docs: 'LibraryDocs' = None
