@@ -1,14 +1,14 @@
 from collections import defaultdict
-from typing import Dict, List, Optional, TYPE_CHECKING
+from typing import Dict, List, Optional, Sequence
 import logging
 import functools
 
 from PySide2.QtCore import Qt
 
+from angrmanagement.ui.views.view import BaseView
+
 from .widgets.qsmart_dockwidget import QSmartDockWidget
 
-if TYPE_CHECKING:
-    from angrmanagement.ui.views.view import BaseView
 
 _l = logging.getLogger(__name__)
 
@@ -38,19 +38,31 @@ class ViewManager:
     def main_window(self):
         return self.workspace._main_window
 
-    def add_view(self, view, caption, category):
+    def _update_view_index_in_category(self, view: BaseView):
+        """
+        Set lowest available index value in category for a view not yet added.
+        """
+        existing_views = self.views_by_category[view.category]
+        if view in existing_views:
+            return
+        existing_ids = {view.index for view in existing_views}
+        max_id = max(existing_ids) if existing_ids else 0
+        candidates = set(range(1, max_id + 2)) - existing_ids
+        view.index = min(candidates)
+
+    def add_view(self, view: BaseView):
         """
         Add a view to this workspace.
 
         :param view:            The view to add.
-        :param str caption:     The caption of the view.
-        :param str category:    The category of the view.
         :return:                None
         """
+        self._update_view_index_in_category(view)
+        self.views_by_category[view.category].append(view)
 
-        self.views_by_category[category].append(view)
-
-        dock = QSmartDockWidget(caption, parent=view, on_close=functools.partial(self.remove_view, view), on_raise=functools.partial(self._handle_raise_view, view))
+        dock = QSmartDockWidget(view.caption, parent=view,
+                                on_close=functools.partial(self.remove_view, view),
+                                on_raise=functools.partial(self._handle_raise_view, view))
         dock_area = self.DOCKING_POSITIONS.get(view.default_docking_position, Qt.RightDockWidgetArea)
         if view.default_docking_position == 'center':
             self.main_window.central_widget.addDockWidget(dock_area, dock)
@@ -68,7 +80,7 @@ class ViewManager:
         if retab:
             self.tabify_center_views()
 
-    def remove_view(self, view):
+    def remove_view(self, view: BaseView):
         """
         Remove a view from this workspace
 
@@ -104,7 +116,7 @@ class ViewManager:
         if retab:
             self.tabify_center_views()
 
-    def raise_view(self, view):
+    def raise_view(self, view: BaseView):
         """
         Find the dock widget of a view, and then bring that dock widget to front.
 
@@ -121,7 +133,7 @@ class ViewManager:
         dock.raise_()
         view.focusWidget()
 
-    def get_center_views(self):
+    def get_center_views(self) -> Sequence[QSmartDockWidget]:
         """
         Get the right dockable views
 
@@ -135,7 +147,7 @@ class ViewManager:
                     docks.append(dock)
         return docks
 
-    def first_view_in_category(self, category) -> Optional['BaseView']:
+    def first_view_in_category(self, category: str) -> Optional['BaseView']:
         """
         Return the first view in a specific category.
 
@@ -147,15 +159,18 @@ class ViewManager:
             return self.views_by_category[category][0]
         return None
 
-    def current_view_in_category(self, category) -> Optional['BaseView']:
+    def current_view_in_category(self, category: str) -> Optional['BaseView']:
         """
         Return the current in a specific category.
 
         :param str category:    The category of the view.
         :return:                The view.
         """
+        current_tab_id = self.get_current_tab_id()
+        if current_tab_id is None:
+            return None
 
-        current = self.get_center_views()[self.get_current_tab_id()]
+        current = self.get_center_views()[current_tab_id]
         view = self.dock_to_view[current]
         if category.capitalize() in view.caption and view.caption == current.windowTitle():
             return view
@@ -167,23 +182,22 @@ class ViewManager:
 
         :return:    None
         """
-
         center_dockable_views = self.get_center_views()
         for d0, d1 in zip(center_dockable_views, center_dockable_views[1:]):
             self.workspace._main_window.central_widget.tabifyDockWidget(d0, d1)
 
-    def get_current_tab_id(self):
+    def get_current_tab_id(self) -> Optional[int]:
         """
         Get Current Tab ID
 
-        :return:    Tab ID (int)
+        :return:    The current tab ID, or None if no current tab exists in the central view area.
         """
 
         center_dockable_views = self.get_center_views()
         for i in range(1,len(center_dockable_views)+1):
             if center_dockable_views[i-1].visibleRegion().isEmpty() is False:
                 return i-1
-        return 1
+        return None
 
     def next_tab(self):
         """
@@ -193,12 +207,14 @@ class ViewManager:
         """
 
         center_dockable_views = self.get_center_views()
-        currentTab = self.get_current_tab_id()
-        if (currentTab + 1) < len(center_dockable_views):
-            center_dockable_views[currentTab + 1].raise_()
+        current_tab_id = self.get_current_tab_id()
+        if current_tab_id is None:
+            return
+        if (current_tab_id + 1) < len(center_dockable_views):
+            center_dockable_views[current_tab_id + 1].raise_()
         else:
             # Start from 1 again to prevent Index Out Of Range error
-            center_dockable_views[(currentTab + 1) % (len(center_dockable_views))].raise_()
+            center_dockable_views[(current_tab_id + 1) % (len(center_dockable_views))].raise_()
 
     def previous_tab(self):
         """
@@ -207,17 +223,24 @@ class ViewManager:
         :return:    None
         """
 
+        current_tab_id = self.get_current_tab_id()
+        if current_tab_id is None:
+            return
+
         center_dockable_views = self.get_center_views()
-        center_dockable_views[self.get_current_tab_id()-1].raise_()
+        center_dockable_views[current_tab_id - 1].raise_()
 
     @property
-    def current_tab(self):
-        return self.get_center_views()[self.get_current_tab_id()].widget()
+    def current_tab(self) -> Optional['BaseView']:
+        current_tab_id = self.get_current_tab_id()
+        if current_tab_id is None:
+            return None
+        return self.get_center_views()[current_tab_id].widget()
 
-    def _handle_raise_view(self, view):
+    def _handle_raise_view(self, view: BaseView):
         self.workspace.plugins.handle_raise_view(view)
 
-    def handle_center_tab_click(self, index):
+    def handle_center_tab_click(self, index: int):
         center_docks = self.get_center_views()
         dock = center_docks[index]
         view = self.dock_to_view[dock]
