@@ -9,7 +9,7 @@ class FunctionsView(BaseView):
     def __init__(self, workspace, default_docking_position, *args, **kwargs):
         super(FunctionsView, self).__init__('functions', workspace, default_docking_position, *args, **kwargs)
 
-        self.caption = 'Functions'
+        self.base_caption = 'Functions'
         self._function_table = None  # type: QFunctionTable
         self._status_label = None
 
@@ -24,6 +24,8 @@ class FunctionsView(BaseView):
         self._function_count = None
         self._displayed_function_count = None
 
+        self.reload()
+
     #
     # Public methods
     #
