@@ -3,12 +3,15 @@ from qtconsole.rich_jupyter_widget import RichJupyterWidget
 from qtconsole.inprocess import QtInProcessKernelManager
 from IPython.lib import guisupport
 
+from ..css import CSS
 
 class QIPythonWidget(RichJupyterWidget):
-    def __init__(self, banner=None, namespace=None, *args, **kwargs):
+    def __init__(self, *args, banner=None, namespace=None, **kwargs):
         if banner is not None:
             self.banner = banner
         RichJupyterWidget.__init__(self, *args, **kwargs)
+        self._on_css_updated()
+        CSS.global_css.am_subscribe(self._on_css_updated)
 
         self.kernel_manager = kernel_manager = QtInProcessKernelManager()
         kernel_manager.start_kernel()
@@ -27,6 +30,13 @@ class QIPythonWidget(RichJupyterWidget):
 
         self.exit_requested.connect(stop)
 
+    def __del__(self):
+        super().__del__()
+        CSS.global_css.am_unsubscribe(self._on_css_updated)
+
+    def _on_css_updated(self):
+        self.setStyleSheet(CSS.global_css.am_obj)
+
     def push_namespace(self, namespace):
         self.kernel_manager.kernel.shell.push(namespace)
 
@@ -38,4 +48,3 @@ class QIPythonWidget(RichJupyterWidget):
 
     def execute_command(self, command):
         self._execute(command, False)
-
