@@ -93,6 +93,7 @@ def get_block_objects(disasm, nodes, func_addr):
         if insn_addr != func_addr and insn_addr in disasm.kb.labels:
             lst.append(Label(insn_addr, get_label_text(insn_addr, disasm.kb)))
         lst.append(disasm.raw_result_map['instructions'][insn_addr])
+        lst.extend(disasm.raw_result_map['ir'][insn_addr])
 
     # initial label, if there is any
     # FIXME: all labels should be generated during CFG recovery, and this step should not be necessary.
@@ -177,8 +178,18 @@ def should_display_string_label(cfg, insn_addr, project):
     return False
 
 
+def is_printable(ch):
+    return 32 <= ch < 127
+
+
 def filter_string_for_display(s):
-    return s.replace("\r", "\\r").replace("\n", "\\n").replace("\t", "\\t")
+    output = ""
+    for ch in s.replace("\r", "\\r").replace("\n", "\\n").replace("\t", "\\t"):
+        char = ord(ch)
+        if not is_printable(char):
+            ch = "\\x%0.2x" % char
+        output += ch
+    return output
 
 
 def get_string_for_display(cfg, insn_addr, project, max_size=20) -> Optional[str]:
@@ -200,8 +211,10 @@ def get_string_for_display(cfg, insn_addr, project, max_size=20) -> Optional[str
                 str_content = next_level.content.decode('utf-8')
 
     if str_content is not None:
-        if len(str_content) > max_size: return '"' + filter_string_for_display(str_content[:max_size]) + '..."'
-        else: return '"' + filter_string_for_display(str_content) + '"'
+        if len(str_content) > max_size:
+            return '"' + filter_string_for_display(str_content[:max_size]) + '..."'
+        else:
+            return '"' + filter_string_for_display(str_content) + '"'
     else:
         return None
 
